
#include <Wire.h> 
#include <EEPROM.h>
#include <LiquidCrystal_I2C.h> //https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads  > "Download repository" rename folder to "LiquidCrystal"
//FIND & SET THE LCD I2C ADDRESS !! > http://playground.arduino.cc/Main/I2cScanner
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE); // 16 X 2 LCD
//GPIO05 (A5) > SCL
//GPIO04 (A4) > SDA

#include <NTPtimeESP.h>

#include <ESP8266WiFi.h>
#include <ESP8266Ping.h>

//define connection credentials
const char* ssid     = "*****";
const char* password = "*****";
const char* ntpServer = "pool.ntp.org"; //ping host address
NTPtime NTPch(ntpServer); // find clocest NTP server at http://www.pool.ntp.org/zone/europe

//define time variables
byte actualHour;
byte actualMinute;
byte actualSecond; 
int actualYear;
byte actualMonth;
byte actualDay;
byte actualDayofWeek;
strDateTime dateTime;

//button definitions
const int buttonSetPin = 14;
#define HOLD_SHORT 1000UL
#define HOLD_LONG 4000UL
unsigned long buttonTimer = 0;
bool buttonActive = false;
bool longPressActive = false; //0 = reset, 1 = short hold, 2 = long hold

//backlight definitions
const int backlightPin = 13;
int backlightPWMVal;
bool glypSync = false;
int eepromAddress = 1;

//define arrays for LCD
char* dayNames[7] = { "Pon", "Tor", "Sre", "Cet", "Pet", "Sob", "Ned" };
char* monthNames[12] = { "Jan", "Feb", "Mar", "Apr", "Maj", "Jun", "Jul", "Avg", "Sep", "Okt", "Nov", "Dec" };
int backlightPWMPreset[] = { 20, 230, 1024 };

//other variables
int countSynchronizations = 0; //counter for failed synchronizaton

//define glyps
byte progress[8] = 
{
  0b00000,
  0b00000,
  0b11111,
  0b11111,
  0b11111,
  0b00000,  
  0b00000,  
  0b00000  
};
byte sync[8] = 
{
  0b10101,
  0b01110,
  0b00100,
  0b00100,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};
byte syncErr[8] = 
{
  0b10000,
  0b10000,
  0b10000,
  0b00000,
  0b10000,
  0b00000,
  0b00000,
  0b00000
};
byte wifi[8] = 
{
  0b10101,
  0b01110,
  0b00100,
  0b00100,
  0b00000,
  0b00100,
  0b01110,
  0b00000
};
byte host[8] = 
{
  0b00100,
  0b01110,
  0b00000,
  0b10101,
  0b01110,
  0b00100,
  0b00100,
  0b00000
};
/*
byte alarm[8] = 
{
  0b00100,
  0b01110,
  0b01110,
  0b01110,
  0b11111,
  0b00000,
  0b00100,
  0b00000
};
*/


void setup() {
  Serial.begin(115200);
  EEPROM.begin(512);
  lcd.begin(16,2);
  lcd.clear();
  lcd.createChar(1, progress);
  lcd.createChar(2, sync); 
  lcd.createChar(3, syncErr);
  lcd.createChar(4, wifi);
  lcd.createChar(5, host);   
//  lcd.createChar(5, alarm);

  pinMode(buttonSetPin, INPUT); 
  pinMode(backlightPin, OUTPUT);

  backlightPWMVal = EEPROM.read(eepromAddress); //read backlight value from eeprom
  yield();
  analogWrite(backlightPin, backlightPWMVal[backlightPWMPreset]);
}


void syncClock() {

  Serial.println("\nConnecting to AP...");
  lcd.clear();

  lcd.setCursor(0, 0);  //draw AP icon
  lcd.write(4);
  lcd.setCursor(2, 0);
  lcd.print("nastavljam uro");
  
  //connecting to ssid
  WiFi.mode(WIFI_STA); //act ESP as client (WiFi.mode(WIFI_AP_STA) = act ESP as AP)
  WiFi.hostname("ESP07-NTPclock");
  WiFi.begin (ssid, password);
  int countOffline = 0;

  while (WiFi.status() != WL_CONNECTED) 
  {
    //draw progress bar
    for (int i = 0; i <= 19; i++) 
    {
      lcd.setCursor(i, 1); //draw progress bar
      lcd.write(1); 
      int j = i-4; //celar progress bar
      if (j >= 0)
      {
        lcd.setCursor(j, 1);
        lcd.print(" ");
      }
    //end draw progress bar

    //limit connection time
      countOffline++;
      Serial.println(countOffline);
      if (countOffline >= 20*5) //if connection canot be established in 20 seconds then skip time sinchronization
      {
        glypSync = false;
        lcd.clear();
        return; //exit from this function & go to loop()
      } 
      //end limit connection time
    
    actualSecond = countOffline/5; //attach lost second during sync for offline mode

    delay(200);
    
    } //end for statement
    
  } //end while connecting
 
  Serial.println(String("Connected to: ") + ssid);
  Serial.println(String("Connecting to NTP server... ") + actualSecond);

     lcd.setCursor(0, 0); //draw host icon
     lcd.write(5);
     
    //check if can reach server to sync clock else skip synchronization.    
     if(Ping.ping(ntpServer, 2)) //ping once takes 20ms, twice=1s, default(4)=4s
     {
       
       // first parameter: Time zone in floating point (for India); second parameter: 1 for European summer time; 2 for US daylight saving time (not implemented yet)
       dateTime = NTPch.getNTPtime(1.0, 1);
       NTPch.printDateTime(dateTime);
       actualHour = dateTime.hour;
       actualMinute = dateTime.minute;
       actualSecond = dateTime.second;
       actualYear = dateTime.year;
       actualMonth = dateTime.month;
       actualDay = dateTime.day;
       actualDayofWeek = dateTime.dayofWeek;
  
       glypSync = true;
       countSynchronizations = 0; //reset counter for failed synchronization
      } 
      else 
      {
        Serial.println("Cannot reach NTP server! Synchronization skipped.");
        actualSecond = countOffline + 1; //add lost second during unsuccessful sync 
        glypSync = false;
      }
    //end sync clock
     
    WiFi.mode(WIFI_OFF);  // WiFi.disconnect or WiFi.softAPdisconnect also reset currently used SSID / password
    lcd.clear(); 
}



void loop() {

//defime at which time clock will be synchronized
  if ( (actualHour == 00) && (actualMinute == 00) && (actualSecond == 00) )
  {
    syncClock();
  }

  
//clock count
unsigned long timerNow = millis();
static unsigned long timerPrev = 0;
  if (timerNow >= timerPrev + 1000)
  {
    timerPrev = timerNow;
    actualSecond++;
  
    if (actualSecond > 59)
    {
      actualSecond = 0;
      actualMinute++;
    }
    if (actualMinute > 59)
    {
      actualMinute = 0;
      actualHour++;
    }
    if (actualHour > 23)
    {
      actualHour = 0;
    }

//clock & button long press print
    if (longPressActive == false) //if not LONG PRESS for reboot
    {
      lcd.setCursor(4, 0);
      if (actualHour < 10)
      {
        lcd.print("0");
      }
      lcd.print(actualHour);
      lcd.print(":");
      if (actualMinute < 10)
      {
        lcd.print("0");
      }
      lcd.print(actualMinute);
      lcd.print(":");
      if (actualSecond < 10)
      {
        lcd.print("0");
      }
      lcd.print(actualSecond);
    }
    else //if press button fore 
    {
      lcd.setCursor(2, 0);
      lcd.print("nastavim uro? ");
    }
//end clock
 
 
//start date
    lcd.setCursor(3, 1);
    
 //   lcd.print(dayNames[actualDay-1]); 
 //   lcd.print(" ");
    
    if (actualDay < 10)
    {
      lcd.print("0");
    }
    lcd.print(actualDay);
    lcd.print(".");

    //lcd.print(monthNames[actualMonth-1]);
    if (actualMonth < 10)
    {
      lcd.print("0");
    }
    lcd.print(actualMonth);
    lcd.print(".");
    
    lcd.print(actualYear);
  //end datum  

    Serial.print(actualHour);   Serial.print(":"); Serial.print(actualMinute);   Serial.print(":"); Serial.print(actualSecond); Serial.print(" | ");
    Serial.print(actualDay);   Serial.print("."); Serial.print(actualMonth); Serial.println(String(".") + actualYear);



//in case of failed time synchronization avoid displaying old date & retry syncing
    if ( glypSync == false)
    {
      lcd.setCursor(0, 1);
      lcd.print("     !sync!     ");

      if ( (actualSecond == 0) && (countSynchronizations < 2) ) //retry to sync 2 more times if default sync failed
      {
        countSynchronizations ++;
        syncClock(); 
      } 
    }
//end failed time synchronization  

  }    


//////////////////////////////////////////////////////////////////////////////////////
//read button state

 if (digitalRead(buttonSetPin) == HIGH) 
 {

   if (buttonActive == false) 
   {
      buttonActive = true;
      buttonTimer = millis(); //start timer for long press 
    }

  //hold button for short time    
    if ((millis() - buttonTimer > HOLD_SHORT) && (longPressActive == false)) 
    {
      longPressActive = true;
      //this will execute only once when hold press
    }

  //hold button for long time    
    if ((millis() - buttonTimer > HOLD_LONG) && (longPressActive == true)) 
    {
      syncClock();
    }
  }
  else
  {
    if (buttonActive == true) 
    {
      if (longPressActive == true) 
      {
        longPressActive = false;

        //clear lcd 1st line
        lcd.setCursor(2, 0);
        lcd.print("              ");
      }
      else 
      {

    //short press - on release
        //change backlight
        backlightPWMVal++; //each press read next value from array
        if(backlightPWMVal >= 3) //max number of presets
        {
          backlightPWMVal = 0;
        }

        //write EEPROM 
        EEPROM.write(eepromAddress, backlightPWMVal); //store backlight value to eeprom
        EEPROM.commit();
        yield();
        analogWrite(backlightPin, backlightPWMVal[backlightPWMPreset]); //set array value to backlight
      }
    //end short press - on release
      
      buttonActive = false;
    }
  }
//end read button state

}

